/*
    Copyright (C) 2011 Harald Sitter <sitter@kde.org>
    Copyright (C) 2011 vlc-phonon AUTHORS

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "LibVLC.h"

#include <QtCore/QByteArray>
#include <QtCore/QCoreApplication>
#include <QtCore/QDebug>
#include <QtCore/QDir>
#include <QtCore/QLibrary>
#include <QtCore/QString>
#include <QtCore/QStringBuilder>
#include <QtCore/QStringList>
#include <QtCore/QVarLengthArray>

#include <vlc/libvlc.h>

namespace CuteVlc {

class LibVLCPrivate
{
public:
    LibVLCPrivate(LibVLC *q) :
        q_ptr(q)
    {
    }

    ~LibVLCPrivate()
    {
    }

    bool init(const QList<QByteArray> &arguments_)
    {
        V_Q(LibVLC);

        if (q->m_vlcInstance) {
            libvlc_release(q->m_vlcInstance);
            q->m_vlcInstance = nullptr;
        }

        QList<QByteArray> arguments = arguments_;
        int size = arguments.size();
        QVarLengthArray<const char *, 64> vlcArgs(size);
        for (int i = 0; i < size; ++i) {
            vlcArgs[i] = arguments.at(i).constData();
        }

        q->m_vlcInstance = libvlc_new(vlcArgs.size(), vlcArgs.constData());
        if (!q->m_vlcInstance) {
            qCritical("libvlc exception: %s", libvlc_errmsg());
            return false;
        }
        return true;
    }

private:
    LibVLC *const q_ptr;
    V_DECLARE_PUBLIC(LibVLC)

    mutable QString libraryPath;
};

LibVLC::LibVLC() :
    d_ptr(new LibVLCPrivate(this)),
    m_vlcInstance(nullptr)
{
    V_D(LibVLC);
    d->init(QList<QByteArray>());
}

LibVLC::~LibVLC()
{
    libvlc_release(m_vlcInstance);
    delete d_ptr;
}

LibVLC &LibVLC::instance()
{
    static LibVLC i;
    return i;
}

bool LibVLC::init(const QList<QByteArray> &arguments)
{
    LibVLC &self = LibVLC::instance();
    LibVLCPrivate *const d = self.d_func();
    return d->init(arguments);
}

const char *LibVLC::errorMessage()
{
    return libvlc_errmsg();
}

} // namespace QtVlc
